# partial unit test for gmpy 0.6 mpf functionality
# relies on Tim Peters' "doctest.py" test-driver
# test-version 0.6
r'''
>>> dir(a)
['_copy', 'binary', 'ceil', 'digits', 'f2q', 'floor', 'getprec', 'getrprec', 'qdiv', 'reldiff', 'setprec', 'sign', 'trunc']
>>>
'''

import gmpy as _g, doctest,sys
__test__={}
a=_g.mpf('123.456')
b=_g.mpf('789.123')

__test__['elemop']=\
r'''
>>> a+b
mpf('9.12579e2')
>>> a-b
mpf('-6.65667e2')
>>> a*b
mpf('9.7421969088e4')
>>> a/b
mpf('1.56447093799065544915e-1')
>>> b+a
mpf('9.12579e2')
>>> b-a
mpf('6.65667e2')
>>> b*a
mpf('9.7421969088e4')
>>> b/a
mpf('6.39193720839813374806e0')
>>> -a
mpf('-1.23456e2')
>>> abs(-a)
mpf('1.23456e2')
>>> _g.fsign(b-a)
1
>>> _g.fsign(b-b)
0
>>> _g.fsign(a-b)
-1
>>> a.sign()
1
>>> (-a).sign()
-1
>>> z=b-b; z.sign()
0
>>> import math
>>> math.ceil(a)
124.0
>>> a.ceil()
mpf('1.24e2')
>>> _g.ceil(a)
mpf('1.24e2')
>>> math.floor(a)
123.0
>>> a.floor()
mpf('1.23e2')
>>> _g.floor(a)
mpf('1.23e2')
>>> a.trunc()
mpf('1.23e2')
>>> _g.trunc(a)
mpf('1.23e2')
>>> x=-a
>>> math.floor(x)
-124.0
>>> x.floor()
mpf('-1.24e2')
>>> _g.floor(x)
mpf('-1.24e2')
>>> x.ceil()
mpf('-1.23e2')
>>> math.ceil(x)
-123.0
>>> _g.ceil(x)
mpf('-1.23e2')
>>> x.trunc()
mpf('-1.23e2')
>>> _g.trunc(x)
mpf('-1.23e2')
>>> _g.ceil(12.3)==math.ceil(12.3)
1
>>> _g.floor(12.3)==math.floor(12.3)
1
>>> _g.ceil(-12.3)==math.ceil(-12.3)
1
>>> _g.floor(-12.3)==math.floor(-12.3)
1
>>> (a**2).reldiff(float(a)**2) < (2.0**-(a.getrprec()-1))
1
>>> (a**2).reldiff(a*a) < (2.0**-(a.getprec()-1))
1
>>> (b**2).reldiff(float(b)**2) < (2.0**-(b.getrprec()-1))
1
>>> (b**2).reldiff(b*b) < (2.0**-(b.getprec()-1))
1
>>>
'''

__test__['cmpr']=\
r'''
>>> c=_g.mpf(a)
>>> c is a
1
>>> c==a
1
>>> c>a
0
>>> c<a
0
>>> d=a._copy()
>>> a is d
0
>>> a == d
1
>>> cmp(a,c)
0
>>> cmp(a,b)
-1
>>> a>b
0
>>> a<b
1
>>> not _g.mpf(0)
1
>>> not a
0
>>> coerce(a,1)
(mpf('1.23456e2'), mpf('1.e0'))
>>> coerce(1,a)
(mpf('1.e0'), mpf('1.23456e2'))
>>> coerce(a,1.0)
(mpf('1.23456e2'), mpf('1.e0'))
>>> a.f2q(0.1)
mpz(123)
>>> a.f2q(0.01)
mpz(123)
>>> a.f2q(0.001)
mpq(247,2)
>>> a.f2q(0.0001)
mpq(1358,11)
>>> a.f2q(0.00001)
mpq(7037,57)
>>> a.f2q(0.000001)
mpq(15432,125)
>>> a.f2q(0.0000001)
mpq(15432,125)
>>> a.f2q()
mpq(15432,125)
>>>
'''

__test__['format']=\
r'''
>>> str(a)
'123.456'
>>> repr(a)
"mpf('1.23456e2')"
>>> _g.set_tagoff(0)
1
>>> a
gmpy.mpf('1.23456e2')
>>> _g.set_tagoff(1)
0
>>> a.digits(10,0)
'1.23456e2'
>>> a.digits(10,1)
'1.e2'
>>> a.digits(10,2)
'1.2e2'
>>> a.digits(10,3)
'1.23e2'
>>> a.digits(10,4)
'1.235e2'
>>> a.digits(10,5)
'1.2346e2'
>>> a.digits(10,6)
'1.23456e2'
>>> a.digits(10,7)
'1.23456e2'
>>> a.digits(10,8)
'1.23456e2'
>>> junk=_g.set_fcoform(14)
>>> frmt=_g.set_fcoform(14)
>>> frmt
'%.14e'
>>> _g.mpf(3.4)
mpf('3.4e0')
>>> print _g.mpf(3.4)
3.4
>>> for i in range(11,99):
...     assert str(_g.mpf(i/10.0))==str(i/10.0)
...
>>>
'''

__test__['binio']=\
r'''
>>> ba=a.binary()
>>> len(ba)
14
>>> for i in range(len(ba)):
...     print ord(ba[i]),
...     if i==len(ba)-1: print
...
0 1 0 0 0 123 116 188 106 126 249 219 34 209
>>> na=(-a).binary()
>>> na[0] == chr(ord(ba[0])|1)
1
>>> for bd,nd in zip(ba[1:],na[1:]):
...    assert bd==nd
>>> _g.fbinary(0)
'\004'
>>> _g.fbinary(0.5)
'\000\000\000\000\000\200'
>>> _g.fbinary(-0.5)
'\001\000\000\000\000\200'
>>> _g.fbinary(-2.0)
'\001\001\000\000\000\002'
>>> _g.fbinary(2.0)
'\000\001\000\000\000\002'
>>> prec=_g.set_minprec(0)
>>> junk=_g.set_minprec(prec)
>>> a.getrprec()==prec
1
>>> b.getrprec()==prec
1
>>> _g.mpf(1.0).getrprec()==prec
1
>>>
'''

print "Unit tests for gmpy 0.6 pre-alpha (mpf functionality)"
print "    running on Python",sys.version
print
print "Testing gmpy %s (GMP %s) with default caching (%s, %s, %s..%s)" % (
    (_g.version(), _g.gmp_version(), _g.get_zcache(), _g.get_qcache(),
    ) + _g.get_zconst())
doctest.testmod(report=0)

print
print "Overall results:"
doctest.master.summarize(1)

